@@ -9,7 +9,7 @@ use Carp;
 use Catmandu::Error;
 use URI;
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 with 'Catmandu::Importer';
 
@@ -186,7 +186,6 @@ sub _map_identify {
     };
 }
 
-
 sub _map_record {
     my ($self, $rec) = @_;
 
@@ -216,7 +215,7 @@ sub _map_record {
     $data;
 }
 
-sub _args_for_records {
+sub _args {
     my $self = $_[0];
 
     my %args = (
@@ -234,21 +233,6 @@ sub _args_for_records {
     return %args;
 }
 
-sub _args {
-    my $self = $_[0];
-
-    my %args;
-
-    if( $self->listSets() ){
-
-    }
-    else{
-        %args = $self->_args_for_records();
-    }
-
-    %args;
-}
-
 sub _verb {
     my $self = $_[0];
 
@@ -312,7 +296,7 @@ sub _retry {
 
         $res = $sub->();
 
-        if ($res->is_error) {
+        if ($res->is_error && ref($res) ne 'HTTP::OAI::Response') {
 
             my $max_retries = $self->max_retries();
             my $_retried = $self->_retried();
@@ -328,7 +312,7 @@ sub _retry {
                 $self->_retried( $_retried );
                 next;
             }
-            else{
+            else {
                 my $err_msg = $self->url . " : " . $res->message." (stopped after ".$self->_retried()." retries)";
                 $self->log->error( $err_msg );
                 Catmandu::Error->throw( $err_msg );
@@ -343,9 +327,11 @@ sub _retry {
 
 sub _list_records {
     my $self = $_[0];
+    my $args = $_[1];
     sub {
         state $stack = [];
         state $resumptionToken = $self->resumptionToken;
+        state $resumptionData  = {};
         state $done  = 0;
 
         my $fill_stack = sub {
@@ -353,7 +339,7 @@ sub _list_records {
         };
 
         if (@$stack <= 1 && $done == 0) {
-            my %args = $self->_args;
+            my %args = $args ? %$args : $self->_args;
 
             # Use the resumptionToken if one found on the last run, or if it was
             # undefined (last record)
@@ -369,6 +355,11 @@ sub _list_records {
             my $res = $self->_retry( $sub );
             if (defined $res->resumptionToken) {
                 $resumptionToken = $res->resumptionToken->resumptionToken;
+
+                $resumptionData->{token}            = $resumptionToken;
+                $resumptionData->{expirationDate}   = $res->resumptionToken->expirationDate;
+                $resumptionData->{completeListSize} = $res->resumptionToken->completeListSize;
+                $resumptionData->{cursor}           = $res->resumptionToken->cursor;
             }
             else {
                 $resumptionToken = undef;
@@ -381,14 +372,24 @@ sub _list_records {
 
         if (my $rec = shift @$stack) {
             if ($rec->isa('HTTP::OAI::Record')) {
-                return $self->_map_record($rec);
+                my $rec = $self->_map_record($rec);
+
+                $rec->{_resumptionToken} = $resumptionToken if defined($resumptionToken);
+                $rec->{_resumption} = $resumptionData if defined($resumptionData);
+
+                return $rec;
             }
             else {
-                return {
+                my $rec =  {
                     _id => $rec->identifier,
                     _datestamp  => $rec->datestamp,
                     _status => $rec->status // "",
-                }
+                };
+
+                $rec->{_resumptionToken} = $resumptionToken if defined($resumptionToken);
+                $rec->{_resumption} = $resumptionData if defined($resumptionData);
+
+                return $rec;
             }
         }
 
@@ -407,8 +408,6 @@ sub _list_sets {
         };
 
         if (@$stack <= 1 && $done == 0) {
-            my %args = $self->_args;
-
             my $sub = sub { $self->oai->ListSets( onRecord => $fill_stack ); };
 
             my $res = $self->_retry( $sub );
@@ -425,6 +424,7 @@ sub _list_sets {
 
 sub _get_record {
     my $self = $_[0];
+    my $args = $_[1];
     sub {
         state $stack = [];
         state $done  = 0;
@@ -434,7 +434,7 @@ sub _get_record {
         };
 
         if (@$stack <= 1 && $done == 0) {
-            my %args = $self->_args;
+            my %args = $args ? %$args : $self->_args;
             my $sub  = sub { $self->oai->GetRecord(%args , onRecord => $fill_stack) };
             my $res  = $self->_retry( $sub );
             $done = 1;
@@ -459,6 +459,7 @@ sub _get_record {
 
 sub _list_metadata_formats {
     my $self = $_[0];
+    my $args = $_[1];
     sub {
         state $stack = [];
         state $done  = 0;
@@ -468,7 +469,7 @@ sub _list_metadata_formats {
         };
 
         if (@$stack <= 1 && $done == 0) {
-            my %args = $self->_args;
+            my %args = $args ? %$args : $self->_args;
             delete $args{metadataPrefix};
 
             my $sub = sub { $self->oai->ListMetadataFormats( %args ); };
